/*
 *  MUSCOD_SUITE/Apps/TEST/Src/SRC/duy2.cpp
 *  (c) Tran Bao Duy, 2019
 *
 * Van der Pol Oscillator (binary variant)
 * https://mintoc.de/index.php/Van_der_Pol_Oscillator_(binary_variant)
 */

#include <cmath>
#include "def_usrmod.hpp"

#define  NMOS   1 /* number of model stages */
#define  NP     0 /* number of parameters */
#define  NRC    0
#define  NRCE   0

#define  NXD    2 /* number of differential states */
#define  NXA    0 /* number of algebraic states */
#define  NU     3 /* number of controls */
#define  NPR    0

#define  NRD_S  2 /* number of decoupled constraints */
#define  NRDE_S 2 /* number of decoupled equality constraints*/

#define NRD_I   4
#define NRDE_I  4

#define c1 -1
#define c2 0.75
#define c3 -2

static void lfcn(double *t, double *xd, double *xa, double *u,
  double *p, double *lval, double *rwh, long *iwh, InfoPtr *info)
{
  *lval = xd[0]*xd[0] + xd[1]*xd[1];
}

static void ffcn(double *t, double *xd, double *xa, double *u,
  double *p, double *rhs, double *rwh, long *iwh, InfoPtr *info)
{
  rhs[0] = xd[1];
  rhs[1] = c1*u[0]*(1-xd[0]*xd[0])*xd[1]
	   + c2*u[1]*(1-xd[0]*xd[0])*xd[1]
	   + c3*u[2]*(1-xd[0]*xd[0])*xd[1] - xd[0];
}

static void rdfcn_s(double *ts, double *sd, double *sa, double *u,
  double *p, double *pr, double *res, long *dpnd, InfoPtr *info)
{
  if (*dpnd) {
    *dpnd = RFCN_DPND(0, *sd, 0, *u, 0, 0);
    return;
  }

  res[0] = sd[0] - 1;
  res[1] = sd[1];
}

static void rdfcn_i(double *ts, double *sd, double *sa, double *u, double *p, double *pr, double *res, long *dpnd, InfoPtr *info)
{
  if (*dpnd) { *dpnd = RFCN_DPND(0, *sd, 0, *u, 0, 0); return; }

  res[0] = u[0]*(u[0]-1.0);
  res[1] = u[1]*(u[1]-1.0);
  res[2] = u[2]*(u[2]-1.0);
  res[3] = u[0] + u[1] + u[2] -1;
}

extern "C" void def_model(void)
{
  def_mdims(NMOS, NP, NRC, NRCE);
  def_mstage(
    0,
    NXD, NXA, NU,
    NULL, lfcn,
    0, 0, 0, NULL, ffcn, NULL,
    NULL, NULL
  );
  def_mpc(0, "Start Point", NPR, NRD_S, NRDE_S, rdfcn_s, NULL);
  def_mpc(0, "Interior Point", NPR, NRD_I, NRDE_I, rdfcn_i, NULL);
}
